/**
 * PolicyDB.h: Transforms a policy document into rules in the database
 * Copyright 2005 by Columbia University; all rights reserved
 */

#ifndef POLICYDB_H
#define POLICYDB_H

#include <xercesc/util/PlatformUtils.hpp>
#include "xercesc/util/XMLString.hpp"
#include "xercesc/framework/MemBufInputSource.hpp"
#include <xercesc/framework/MemBufFormatTarget.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/util/PlatformUtils.hpp>


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <iostream>
#include <fstream>
#include <string>
#include "PolicyQuery.h"

XERCES_CPP_NAMESPACE_USE

using namespace std;


typedef struct privacy_rule {
    
    char *presentity;		
    char *rule_id;
    // conditions

    // identity

    // <one>
    char *one_id;
    char *one_scheme;

    // <many>
    char *except_domain;
    char *except_id;
    char *many_domain;
    char *except_scheme;


    // other conditions
    char *sphere;
    string validity_from;
    string validity_to;

    // actions
    char *sub_handling;
    
    //transformations
    //devices
    char *all_devices;
    //services
    char *all_services;
    //persons
    char *all_persons;

    //other permissions
    char *provide_activities;
    char *provide_class;
    char *provide_device_id;
    char *provide_mood;
    char *provide_place_is;
    char *provide_place_type;
    char *provide_privacy;
    char *provide_relationship;
    char *provide_status_icon;
    char *provide_sphere;
    char *provide_time_offset;
    char *provide_user_input;
    char *provide_note;
    char *provide_unknown_attribute;
    char *unknown_attribute_name;

}rule;

class PolicyDB {

    public :
      
        bool parsePolicyDocument(char* doc);
        PolicyDB(char *presentity,char *database,char *machine,char *user,char *password);
        ~PolicyDB();
        
    protected :
        
        void processConditions(DOMNode *conditions,const XMLCh *common_policy_prefix);
        void processActions(DOMNode *actions);
        void processTransformations(DOMNode *transformations);
        void processServicePermission(char *service_type , char* service_value);
        void processDevicePermission(char *device_type ,char* device_value);
        void processPersonPermission(char *person_type ,char* person_value);
        void processRuleset(DOMNode *cur_node,const XMLCh *common_policy_prefix);
        void processRule(DOMNode *rule,const XMLCh *common_policy_prefix);
        string convert_to_sqldatetime(string xmldate); 
	void reset_rule(void);

    private :
          
       
        PolicyQuery *query;
        XercesDOMParser* parser;
        rule *policy_rule;
	ErrorHandler* errhandler;
};

#endif
