/**
 * PolicyQuery.cpp: PolicyQuery.h implementation
 * Copyright 2005 by Columbia University; all rights reserved
 */

#include "PolicyQuery.h"

/* PolicyQuery :: rule_combine(string ruleid_set,char *presentity)
 * Query to combine all rules in the ruleid_set.
 * Returns the resultset.
 */
 
MYSQL_RES* PolicyQuery :: rule_combine(string ruleid_set,char *presentity) {
  
    MYSQL_RES *res;
    char *query = (char *)calloc(1000,sizeof(char));
    sprintf(query,"SELECT MAX(subhandling),BIT_OR(provide_activities),BIT_OR(provide_class),BIT_OR(provide_device_id),BIT_OR(provide_mood),BIT_OR(provide_place_type),BIT_OR(provide_place_is),BIT_OR(provide_privacy),BIT_OR(provide_relationship),BIT_OR(provide_status_icon),BIT_OR(provide_sphere),BIT_OR(provide_time_offset),MAX(provide_user_input),BIT_OR(provide_note),BIT_OR(provide_unknown_attribute),BIT_OR(provide_all_services), BIT_OR(provide_all_devices),BIT_OR(provide_all_persons) FROM ruleset WHERE (ruleid IN (%s)) AND (presentity = '%s')",ruleid_set.c_str(),presentity);
    cout<<"Query :"<<query<<endl;
    res = dbm->read_query(dbc,query);
    free(query);
    return res;
       
}/* PolicyQuery :: rule_combine(string ruleid_set,char *presentity) */


/* PolicyQuery :: getsubscription(string ruleid_set,char *presentity)
 * Query to get the final subscription value based on values present in rules of ruleid_set. 
 * Returns the resultset.
 */

MYSQL_RES* PolicyQuery :: getsubscription(string ruleid_set,char *presentity) {

    MYSQL_RES *res;
    char *query = (char *)calloc(1000,sizeof(char));
    sprintf(query,"SELECT MAX(subhandling) FROM ruleset WHERE (ruleid IN (%s)) AND (presentity = '%s')",ruleid_set.c_str(),presentity);
    cout<<"Query : "<<query<<endl;	
    res = dbm->read_query(dbc,query);
    free(query);
    return res;

}/* PolicyQuery :: getsubscription(string ruleid_set,char *presentity) */
  

/* PolicyQuery :: select_devicepermission(char *presentity,string ruleid_set)
 * Query to select all device_type,device_value for a given presentity and a set of rules in ruleid_set 
 * Returns the resultset
 */

MYSQL_RES* PolicyQuery :: select_devicepermission(char *presentity,string ruleid_set)
{
    MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));
   sprintf(query,"SELECT DISTINCT device_type,device_value FROM devicepermission WHERE (ruleid IN (%s)) AND (presentity = '%s')",ruleid_set.c_str(),presentity);
   cout<<"Query devicepermission: "<<query<<endl; 	
   res = dbm->read_query(dbc,query);
   cout<<"Queried devicepermission: "<<endl; 	
   free(query);
   return res;
}/* PolicyQuery :: select_devicepermission(char *presentity,string ruleid_set) */


/* PolicyQuery :: select_ruleidentity(char *presentity,char *identity,char *domain,char *scheme)
 * Query to select ruleids that match a given watcher identity based on identity
 * Returns the resultset
 */

MYSQL_RES* PolicyQuery :: select_ruleidentity(char *presentity,char *identity,char *domain,char *scheme)
{
   MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));

   sprintf(query,"SELECT DISTINCT ruleid FROM ruleidentity WHERE (presentity = '%s') AND ((one_id = '%s' and scheme = '%s') OR (one_id = '%s' and scheme = '(null)') OR (one_id = '(null)')) AND (except_domain <> '%s' OR except_domain = '(null)') AND ((except_id <> '%s' and scheme <> '%s') OR (except_id <> '%s' and scheme = '(null)') OR (except_id = '(null)')) AND (many_domain = '%s' OR many_domain = '(null)')",presentity,identity,scheme,identity,domain,identity,scheme,identity,domain);
   
   cout<<"Query : "<<query<<endl;	
   res = dbm->read_query(dbc,query);
   free(query);
   return res;
}/* PolicyQuery :: select_ruleidentity(char *presentity,char *identity,char *domain,char *scheme) */


/* PolicyQuery :: select_rulematch(char *presentity,char *ruleid, char *sphere)
 * Query to select ruleids that match a given watcher identity based on sphere and datetime
 * Returns resultset
 */

MYSQL_RES* PolicyQuery :: select_rulematch(char *presentity,char *ruleid,char *sphere)
{
   MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));
   sprintf(query,"SELECT ruleid FROM ruleset WHERE (presentity = '%s') AND (ruleid = '%s') AND (sphere = '%s' OR sphere = '(null)') AND (((TIMEDIFF(NOW(),validity_from) > 0) AND (TIMEDIFF(validity_to,NOW()) > 0)) OR ((validity_from = '0000-00-00 00:00:00') AND (validity_to = '0000-00-00 00:00:00')))",presentity,ruleid,sphere);
   cout<<"Query generated : "<<query<<endl;
   res = dbm->read_query(dbc,query);
   free(query);
   return res;

}/* PolicyQuery :: select_rulematch(char *presentity,char *sphere) */


/* PolicyQuery :: select_personpermission(char *presentity,string ruleid_set)
 * Query to select all person_type,person_value for a given presentity and a set of rules in ruleid_set 
 * Returns the resultset
 */
     
MYSQL_RES* PolicyQuery :: select_personpermission(char *presentity,string ruleid_set)
{
   MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));
   sprintf(query,"SELECT DISTINCT person_type,person_value FROM personpermission WHERE (ruleid IN (%s)) AND (presentity = '%s')",ruleid_set.c_str(),presentity);
   cout<<"Query personpermision: "<<query<<endl; 	
   res = dbm->read_query(dbc,query);
   free(query);
   return res;
}/* PolicyQuery :: select_personpermission(char *presentity,string ruleid_set) */


/* PolicyQuery :: select_servicepermission(char *presentity,string ruleid_set)
 * Query to select all service_type,service_value for a given presentity and a set of rules in ruleid_set 
 * Returns the resultset
 */

MYSQL_RES* PolicyQuery :: select_servicepermission(char *presentity,string ruleid_set)
{
   MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));
   sprintf(query,"SELECT DISTINCT service_type,service_value FROM servicepermission WHERE (ruleid IN (%s)) AND (presentity = '%s')",ruleid_set.c_str(),presentity);
   cout<<"Query servicepermission: "<<query<<endl;	
   if(dbc==NULL)
	cout << "retuned NULL" << endl;
   if(dbc->conn==NULL)
	cout << "retuned NULL conn" << endl;

   res = dbm->read_query(dbc,query);
   free(query);
   return res;
}/* PolicyQuery :: select_servicepermission(char *presentity,string ruleid_set) */


/* PolicyQuery :: insert_ruleidentity(char *presentity,char *ruleid,char *one_id,char *scheme,
 *                  char *except_id,char *except_scheme,char *except_domain,char *many_domain)
 * Query to insert entries into the ruleidentity table
 */

void PolicyQuery :: insert_ruleidentity(char *presentity,char *ruleid,char *one_id,char *scheme,char *except_id,char *except_scheme,char *except_domain,char *many_domain)
{

    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"INSERT INTO ruleidentity VALUES ('%s','%s','%s','%s','%s','%s','%s','%s')",presentity,ruleid,one_id,scheme,except_id,except_scheme,except_domain,many_domain);
    printf("The query generated is \n%s\n",query);
    unsigned long rows = dbm->write_query(dbc,query);
    printf("\nnumber of rows inserted %ld\n",rows);
    free(query);
}/* PolicyQuery :: insert_ruleidentity(char *presentity,char *ruleid,char *one_id,char *scheme,
                   char *except_id,char *scheme,char *except_domain,char *many_domain) */ 


/* PolicyQuery :: insert_ruleset(char *presentity,char *ruleid,char *sphere,char *validity_from,
 *		  char *validity_to,char *subhandling,char *provide_all_services,char *provide_all_devices,
 *                char *provide_all_persons,char *provide_activities,char *provide_class,char *provide_device_id,
 *                char *provide_mood,char *provide_place_type,char *provide_place_is,char *provide_privacy,
 *                char *provide_relationship,char *provide_status_icon,char *provide_sphere,char *provide_time_offset,
 *                char *provide_user_input,char *provide_note,char *provide_unknown_attribute,char *unknown_attribute_name) 
 * Query to insert entries into the ruleset table.
 */
void PolicyQuery :: insert_ruleset(char *presentity,char *ruleid,char *sphere,char *validity_from, char *validity_to,char *subhandling,char *provide_all_services,char *provide_all_devices,char *provide_all_persons,char *provide_activities,char *provide_class,char *provide_device_id,char *provide_mood,char *provide_place_type,char *provide_place_is,char *provide_privacy,char *provide_relationship,char *provide_status_icon,char *provide_sphere,char *provide_time_offset,char *provide_user_input,char *provide_note,char *provide_unknown_attribute,char *unknown_attribute_name)
{
   
    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"INSERT INTO ruleset VALUES ('%s','%s','%s',%s,%s,'%s',%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,'%s',%s,%s,'%s')",presentity,ruleid,sphere,validity_from,validity_to,subhandling,provide_all_services,provide_all_devices,provide_all_persons,provide_activities,provide_class,provide_device_id,provide_mood,provide_place_type,provide_place_is,provide_privacy,provide_relationship,provide_status_icon,provide_sphere,provide_time_offset,provide_user_input,provide_note,provide_unknown_attribute,unknown_attribute_name);
    printf("The query generated is \n%s\n",query);
    unsigned long rows = dbm->write_query(dbc,query); 
    printf("\nnumber of rows inserted %ld\n",rows);
    free(query);
}/* PolicyQuery :: insert_ruleset(char *presentity,char *ruleid,char *sphere,char *validity_from,
		  char *validity_to,char *subhandling,char *provide_all_services,char *provide_all_devices,
                  char *provide_all_persons,char *provide_activities,char *provide_class,char *provide_device_id,
                  char *provide_mood,char *provide_place_type,char *provide_place_is,char *provide_privacy,
                  char *provide_relationship,char *provide_status_icon,char *provide_sphere,char *provide_time_offset,
                  char *provide_user_input,char *provide_note,char *provide_unknown_attribute,char *unknown_attribute_name) */


/* PolicyQuery :: insert_servicepermission(char *presentity,char *ruleid,char *service_type,char *service_value)
 * Query to insert entries into servicepermission table
 */
  
void PolicyQuery :: insert_servicepermission(char *presentity,char *ruleid,char *service_type,char *service_value)
{

    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"INSERT INTO servicepermission VALUES ('%s','%s','%s','%s')",presentity,ruleid,service_type,service_value);
    printf("The query generated is \n*%s*\n",query);
   
    unsigned long rows = dbm->write_query(dbc,query); 
    printf("\nnumber of rows inserted %ld\n",rows);
    free(query);
}/*PolicyQuery :: insert_servicepermission(char *presentity,char *ruleid,char *service_type,char *service_value) */



/* PolicyQuery :: insert_devicepermission(char *presentity,char *ruleid,char *device_type,char *device_value)
 * Query to insert entries into devicepermission table
 */

void PolicyQuery :: insert_devicepermission(char *presentity,char *ruleid,char *device_type,char *device_value)
{

    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"INSERT INTO devicepermission VALUES ('%s','%s','%s','%s')",presentity,ruleid,device_type,device_value);
    printf("The query generated is \n*%s*\n",query);
   
    unsigned long rows = dbm->write_query(dbc,query); 
    printf("\nnumber of rows inserted %ld\n",rows);
    free(query);
}
/* PolicyQuery :: insert_devicepermission(char *presentity,char *ruleid,char *device_type,char *device_value) */


/* PolicyQuery :: insert_personpermission(char *presentity,char *ruleid,char *person_type,char *person_value)
 * Query to insert entries into personpermission table
 */

void PolicyQuery :: insert_personpermission(char *presentity,char *ruleid,char *person_type,char *person_value)
{

    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"INSERT INTO devicepermission VALUES ('%s','%s','%s','%s')",presentity,ruleid,person_type,person_value);
    printf("The query generated is \n%s\n",query);
    unsigned long rows = dbm->write_query(dbc,query); 
    printf("\nnumber of rows inserted %ld\n",rows);
    free(query);
}
/* PolicyQuery :: insert_personpermission(char *presentity,char *ruleid,char *person_type,char *person_value) */


/* PolicyQuery :: verify_presentity(char *presentity)
 * Query to verify if presentity exists in the database
 */

void PolicyQuery :: verify_presentity(char *presentity) {
	
    MYSQL_RES *res;
   /* send SQL query */
   char *query = (char *)calloc(1000,sizeof(char));
   sprintf(query,"SELECT ruleid FROM ruleset WHERE (presentity = '%s')",presentity);
   cout<<"Query for Verify Presentity "<<query<<endl;	
   if(dbc==NULL)
	cout << "retuned NULL" << endl;
   if(dbc->conn==NULL)
	cout << "retuned NULL conn" << endl;

   res = dbm->read_query(dbc,query);
   if(mysql_num_rows(res) > 0) {	
	cout<<"Presentity already exists...Updating database"<<endl;
	cleanup_tables(presentity);
   }
   else 
	cout<<"Presentity does not exist....Adding new presentity"<<endl;		
  
   free(query);
   mysql_free_result(res);
  
}
/* PolicyQuery :: verify_presentity(char *presentity) */


/* PolicyQuery :: cleanup_tables(char *presentity)
 * Deletes all entries pertaining to a presentity from all the tables
 */

void PolicyQuery :: cleanup_tables(char *presentity) {

    char *query = (char *)calloc(1000,sizeof(char));
    memset(query,0,sizeof(query));
    sprintf(query,"DELETE FROM ruleidentity WHERE presentity = '%s'",presentity);
    printf("The query generated is \n%s\n",query);
    unsigned long rows = dbm->write_query(dbc,query);
    printf("\nruleidentity : number of rows affected %ld\n",rows);

    memset(query,0,sizeof(query));
    sprintf(query,"DELETE FROM ruleset WHERE presentity = '%s'",presentity);
    printf("The query generated is \n%s\n",query);
    rows = dbm->write_query(dbc,query); 
    printf("\nruleset : number of rows affected %ld\n",rows);

    memset(query,0,sizeof(query));
    sprintf(query,"DELETE FROM servicepermission WHERE presentity = '%s'",presentity);
    printf("The query generated is \n%s\n",query);
    rows = dbm->write_query(dbc,query);
    printf("\nservicepermission : number of rows affected %ld\n",rows);
   
    memset(query,0,sizeof(query));
    sprintf(query,"DELETE FROM devicepermission WHERE presentity = '%s'",presentity);
    printf("The query generated is \n%s\n",query);
    rows = dbm->write_query(dbc,query); 
    printf("\ndevicepermission : number of rows affected %ld\n",rows);
   
    memset(query,0,sizeof(query));
    sprintf(query,"DELETE FROM personpermission WHERE presentity = '%s'",presentity);
    printf("The query generated is \n%s\n",query);
    rows = dbm->write_query(dbc,query); 
    printf("\npersonpermission : number of rows affected %ld\n",rows);
   
    free(query);


}


/* PolicyQuery :: PolicyQuery(char *database,char *machine,char *user,char *password) {
 * Constructor - Initializes the database configuration object.Gets the DB connection.
 */

PolicyQuery :: PolicyQuery(char *database,char *machine,char *user,char *password) {

    
    struct db_config *dc = new db_config();
    strcpy(dc->server,machine);
    strcpy(dc->user,user);
    strcpy(dc->password,password);
    strcpy(dc->database,database); 
    PolicyQuery :: dbm = DBManager::getInstance();
    PolicyQuery :: dbc = dbm->getConnectionObj();
    
}/* PolicyQuery :: PolicyQuery(char *database,char *machine,char *user,char *password) */
   

/* PolicyQuery :: ~PolicyQuery()
 * Destructor
 */
PolicyQuery :: ~PolicyQuery() {

}/* PolicyQuery :: ~PolicyQuery() */
