/**
 * PrivacyFilter.cpp: PrivacyFilter.h implementation
 * Copyright 2005 by Columbia University; all rights reserved
 */

#include "PrivacyFilter.h"

/* PrivacyFilter :: PrivacyFilter(char *presentity,char *subscriber,char *domain,char *scheme,char *sphere) 
 * Constructor - Gets the Filter object for a given subscriber of a presentity
 */
 
PrivacyFilter :: PrivacyFilter(char *presentity,char *subscriber,char *domain,char *scheme,char *sphere) {
  
  matched_rule = (new PrivacyRuleMatch())->getFilter(presentity,subscriber,domain,scheme,sphere);
  
}/* PrivacyFilter :: PrivacyFilter(char *presentity,char *subscriber,char *domain,char *scheme,char *sphere) */


/* PrivacyFilter :: ~PrivacyFilter()
 * Destructor - Deletes the filter object
 */
 
PrivacyFilter :: ~PrivacyFilter() {
   delete matched_rule;
}/* PrivacyFilter :: ~PrivacyFilter() */

/* PrivacyFilter :: is_node_name(DOMNode *node,const char *name)
 * Checks if a given DOMNode (node) has a name equal to name
 * Returns true if equal else false.
 */

bool PrivacyFilter :: is_node_name(DOMNode *node,const char* name) 
{
  bool ret = false;

  if (node != NULL) {
    const XMLCh *v = node->getLocalName();
    char *n = XMLString::transcode(v);
    if (n != NULL && !strcasecmp(name, n)) {
      ret = true;
    }
    XMLString::release(&n);
  }
  return ret;
}/* PrivacyFilter :: is_node_name(DOMNode *node,const char *name) */

/* PrivacyFilter :: delete_node(DOMNode *node)
 * Removes a node and its children
 */

void PrivacyFilter::delete_node(DOMNode *node) {
   
  DOMNode *parent = node->getParentNode();
        node = parent->removeChild(node);
        node->release();

}/* PrivacyFilter :: delete_node(DOMNode *node) */


/* PrivacyFilter :: serializeDOM(DOMNode *node)
 * Serializes a DOMNode.In this context serializes the filtered presence document   
 */

char *PrivacyFilter :: serializeDOM(DOMNode *node) {

   if(node==NULL)
       cout << "node passed to be serialized is NULL " << endl;

   if(node->getNodeType() == DOMNode::DOCUMENT_NODE){
       cout << "node type is DOCUMENT NODE" << endl;
   }
   const DOMDocument *docu = (node->getNodeType() == DOMNode::DOCUMENT_NODE)?
                            (const DOMDocument*)node : node->getOwnerDocument();
   if(docu==NULL)
      cout << "owner document is NULL" << endl;

    XMLCh tempStr[100];
    XMLString::transcode("LS", tempStr, 99);
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(tempStr);
    DOMWriter* theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();

    if (theSerializer->canSetFeature(XMLUni::fgDOMWRTDiscardDefaultContent, true))
                theSerializer->setFeature(XMLUni::fgDOMWRTDiscardDefaultContent, true);

    if (theSerializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true))
        theSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);

        MemoryManager *const manager = XMLPlatformUtils::fgMemoryManager;
        MemBufFormatTarget *myMemFormTarget = new MemBufFormatTarget(2048,manager);

    try {
           
	theSerializer->writeNode(myMemFormTarget, *node);
    }
    catch (const XMLException& toCatch) {
	char* message = XMLString::transcode(toCatch.getMessage());
        cerr << "Exception message is: \n"<< message << "\n"<< endl;
        XMLString::release(&message);
        return NULL;
    }
    catch (const DOMException& toCatch) {
        char* message = XMLString::transcode(toCatch.msg);
        cerr <<"Exception message is: \n" << message << "\n"<< endl;
        XMLString::release(&message);
        return NULL;
    }
    catch (...) {
        cout<<"Unexpected Exception \n" ;
        return NULL;
    }

    char * text= (char *)myMemFormTarget->getRawBuffer();
   
    cout<<"\nSerialized XML is "<<text<<endl;
    theSerializer->release();
    return text;
}/*PrivacyFilter :: serializeDOM(DOMNode *node) */


/* PrivacyFilter :: apply_filter(DOMNode *node)
 * Applies filter for a node based on appropriate value present in the filter object
 */

void PrivacyFilter :: apply_filter(DOMNode *node) {

    //  multimap service_multimap = matched_rule->service_permissions;
    //  multimap device_mulimap = matched_rule->device_permissions;
    //  multimap person_multimap = matched_rule->person_permissions;   
        	 
    if (is_node_name(node,"tuple")) {
	
	
        // check for provide-services     	
    }

    if (is_node_name(node,"person")) {

       // check for provide-person
    }

    if (is_node_name(node,"device")) {
    
       // check for provide-device
    } 
 	
    if (is_node_name(node,"activities")) {
        if((matched_rule->provide_activities == false)) 
            delete_node(node);
    }
    
    if (is_node_name(node,"class")) {
        if(matched_rule->provide_class == false)
            delete_node(node);
    }

    if (is_node_name(node,"deviceID")) {
        if((matched_rule->provide_device_id == false) && (is_node_name(node->getParentNode(),"tuple"))) { 
            delete_node(node);
	}
    }
    
    if (is_node_name(node,"mood")) {
        if(matched_rule->provide_mood == false) {
            delete_node(node);
        }
    }
    
    if (is_node_name(node,"place-type")) {
        if(matched_rule->provide_place_type == false)
            delete_node(node);
    }

    if (is_node_name(node,"place-is")) {
        if(matched_rule->provide_place_is == false)
            delete_node(node);
    }

    if (is_node_name(node,"privacy")) {
        if(matched_rule->provide_privacy == false) {
           delete_node(node);
        }
    }

    if (is_node_name(node,"relationship")) {
        if(matched_rule->provide_relationship == false)
            delete_node(node);
    } 
     
    if (is_node_name(node,"status-icon")) {
        if(matched_rule->provide_relationship == false)
            delete_node(node);
    } 

    if (is_node_name(node,"sphere")) {
        if(matched_rule->provide_sphere == false) 
            delete_node(node);
    }

    if (is_node_name(node,"time-offset")) {
        if(matched_rule->provide_time_offset == false) 
            delete_node(node);
    }

    if (is_node_name(node,"user-input")) {
	// false
        if(matched_rule->provide_user_input == 0) {
 	    delete_node(node);
	}
	// bare
	else if(matched_rule->provide_user_input == 1) {
            // remove idle-threshold and since
	    if (((DOMElement *)node)->hasAttribute(XMLString::transcode("idle-threshold"))) {
	        DOMAttr *idle_threshold = ((DOMElement *)node)->getAttributeNode(XMLString::transcode("idle-threshold"));
		((DOMElement *)node)->removeAttributeNode(idle_threshold);
		idle_threshold->release();
	
            }
	     
	    if (((DOMElement *)node)->hasAttribute(XMLString::transcode("last-input"))) {
	        DOMAttr *last_input = ((DOMElement *)node)->getAttributeNode(XMLString::transcode("last-input"));
		((DOMElement *)node)->removeAttributeNode(last_input);
		last_input->release();
	    } 
	}
	// threshold
	else if(matched_rule->provide_user_input == 2) {
	   // retain only idle-threshold  
            
	    if (((DOMElement *)node)->hasAttribute(XMLString::transcode("last-input"))) {
	        DOMAttr *last_input = ((DOMElement *)node)->getAttributeNode(XMLString::transcode("last-input"));
		((DOMElement *)node)->removeAttributeNode(last_input);
		last_input->release();
	    }
	}
    }
    
    if (is_node_name(node,"note")) {
        if(matched_rule->provide_note == false) 
            delete_node(node);
    }
    
}/* PrivacyFilter :: apply_filter(DOMNode *node) */    
   

/* PrivacyFilter :: doFilter(DOMDocument *doc) 
 * API for filtering a DOMDocument doc (presence document)
 * Returns the document(root node) after filtering 
 */

DOMNode *PrivacyFilter :: doFilter(DOMDocument *doc) {
   
   
    DOMNode *root = doc->getDocumentElement(); 
    DOMNodeIterator *iterator = doc->createNodeIterator(root,1,NULL,false);
    DOMNode *cur_node = iterator->getRoot();	
  	
    while(cur_node != NULL) {
       
       if(strcmp(XMLString::transcode(cur_node->getNodeName()),"#text") != 0) {		
	 cout<<"Processing :  "<<XMLString::transcode(cur_node->getLocalName())<<endl;	
         apply_filter(cur_node);
	  
       }
       cur_node = iterator->nextNode();
    }
    return root;	
    iterator->release();	
}/* PrivacyFilter :: doFilter(DOMDocument *doc) */

