/**
 * PrivacyRuleMatch.cpp: PrivacyRuleMatch.h implementation
 * Copyright 2005 by Columbia University; all rights reserved
 */

#include "PrivacyRuleMatch.h"

/* PrivacyRuleMatch :: PrivacyRuleMatch()
 * Constructor - Creates new query and filter objects
 */

PrivacyRuleMatch :: PrivacyRuleMatch() {

    query = new PolicyQuery("test","metro-north.cs.columbia.edu","root","tiger");
    filter =(final_rule *)malloc(sizeof(final_rule));
}/* PrivacyRuleMatch :: PrivacyRuleMatch() */

/* PrivacyRuleMatch :: ~PrivacyRuleMatch()
 * Destructor - Deletes query object
 */

PrivacyRuleMatch :: ~PrivacyRuleMatch() {

    delete query;
	
}/* PrivacyRuleMatch :: ~PrivacyRuleMatch() */ 

/* PrivacyRuleMatch :: convert_to_boolean(char *str)
 * Converts a string "0" to false and "1" to true
 * Returns boolean value
 */

bool PrivacyRuleMatch :: convert_to_boolean(char *str) {
   
    if(strcmp(str,"0") == 0) {
        return false;
    }
    else {
        return true;
    }
        
}/* PrivacyRuleMatch :: convert_to_boolean(char *str) */


/* PrivacyRuleMatch :: convert_list_to_string(list<const char *> ruleid_list)
 * Converts a list form of ruleids to string format
 * Returns string format of ruleid_list
 */ 

string PrivacyRuleMatch :: convert_list_to_string(list<const char *> ruleid_list) {
    
    string ruleid_set = "";
    list<const char *>::iterator ruleid_iterator ;
    for(ruleid_iterator = ruleid_list.begin(); ruleid_iterator != ruleid_list.end(); ++ruleid_iterator) {
        
        ruleid_set += "\'";
        ruleid_set += *ruleid_iterator;
        ruleid_set.append("\'");
        ruleid_set.append(",");
    }
 
    // Remove last comma 
    ruleid_set.erase(ruleid_set.size()-1);
    cout<<"List of ruleids string : "<<ruleid_set<<endl;    
    return ruleid_set;
    
}/* PrivacyRuleMatch :: convert_list_to_string(list<const char *> ruleid_list) */ 

/* PrivacyRuleMatch :: getFilter(char *presentity, char *watcher, char *domain, char *scheme ,char *sphere) 
 * Gets matching rules for the watcher ,combines them and assigns the values to the filter object elements
 * Returns the filter object
 */
 
final_rule * PrivacyRuleMatch :: getFilter(char *presentity, char *watcher, char *domain, char *scheme ,char *sphere) {
    
   MYSQL_ROW rule_row;
   list<const char *> ruleid_list;
   ruleid_list = getRuleidList(presentity,watcher,domain,scheme,sphere);
    
   if (ruleid_list.size() != 0) {
      
      MYSQL_RES *combined_result = query->rule_combine(convert_list_to_string(ruleid_list),presentity);
      if(((rule_row = mysql_fetch_row(combined_result)) != NULL) && (mysql_num_rows(combined_result) == 1)) {
            
            printf("Starting to combine matching rules\n");
	   
            // Subcription not actually required because it is already done in 
            // SubscriptionAuthorization Module 

            char *sub = rule_row[0];
            if(strcmp(sub,"block") == 0) {
                filter->subscription_handling = 0;
            }
            else if(strcmp(sub,"confirm") == 0) {
                filter->subscription_handling = 1;
            }
            else if(strcmp(sub,"polite-block") == 0) {
                filter->subscription_handling = 2;
            }
            else if(strcmp(sub,"allow") == 0) {
                filter->subscription_handling = 3;
            }
            
            filter->provide_activities = convert_to_boolean(rule_row[1]);
            cout<<"Final Activities permission : "<<filter->provide_activities<<endl;
            filter->provide_class = convert_to_boolean(rule_row[2]);
            cout<<"Final Class permission : "<<filter->provide_class<<endl;
            filter->provide_device_id = convert_to_boolean(rule_row[3]);
	    cout<<"Final Device-ID permission  : "<<filter->provide_device_id<<endl;     
            filter->provide_mood = convert_to_boolean(rule_row[4]);
	    cout<<"Final Mood permission : "<<filter->provide_mood<<endl;	
            filter->provide_place_type = convert_to_boolean(rule_row[5]);
	    cout<<"Final Place type permission : "<<filter->provide_place_type<<endl;
            filter->provide_place_is = convert_to_boolean(rule_row[6]);
	    cout<<"Final Place is permission : "<<filter->provide_place_is<<endl;
            filter->provide_privacy = convert_to_boolean(rule_row[7]);
	    cout<<"Final Privacy permission : "<<filter->provide_privacy<<endl;
            filter->provide_relationship = convert_to_boolean(rule_row[8]);
	    cout<<"Final Relationship permission : "<<filter->provide_relationship<<endl;	
            filter->provide_status_icon = convert_to_boolean(rule_row[9]);
	    cout<<"Final Status Icon permission : "<<filter->provide_status_icon<<endl;
            filter->provide_sphere = convert_to_boolean(rule_row[10]);
            cout<<"Final Sphere permission : "<<filter->provide_sphere<<endl;
            filter->provide_time_offset = convert_to_boolean(rule_row[11]);
	    cout<<"Final Time-offset permission : "<<filter->provide_time_offset<<endl;		
            if(strcmp(rule_row[12],"false") == 0) {
	      filter->provide_user_input = 0;
            }
            
            if(strcmp(rule_row[12],"bare") == 0) {
              filter->provide_user_input = 1;
            }
            
            if(strcmp(rule_row[12],"thresholds") == 0) {
	      filter->provide_user_input = 2;
            }
            
            if(strcmp(rule_row[12],"full") == 0) {
	      filter->provide_user_input = 3;
            }
	    cout<<"Final User-Input permission : "<<filter->provide_user_input<<endl;	

            filter->provide_note = convert_to_boolean(rule_row[13]);
	    cout<<"Final Provide-note attribute : "<<filter->provide_note<<endl;  
            filter->provide_unknown_attribute = convert_to_boolean(rule_row[14]);
	    cout<<"Final Provide-unknown attribute : "<<filter->provide_unknown_attribute<<endl;  
	    filter->provide_all_services = convert_to_boolean(rule_row[15]);
	    cout<<"Final Provide all services permission : "<<filter->provide_all_services<<endl;
            filter->provide_all_devices = convert_to_boolean(rule_row[16]);
            cout<<"Final Provide all devices permission : "<<filter->provide_all_devices<<endl;
            filter->provide_all_persons = convert_to_boolean(rule_row[17]);
	    cout<<"Final Provide all persons permission : "<<filter->provide_all_persons<<endl;
      
            printf("Finished combining rules\n");
        }
	
        mysql_free_result(combined_result);
 	cout<<"Going to get device Permissions\n"<<endl;
        getDevicePermissions(presentity,convert_list_to_string(ruleid_list));
        cout<<"Going to get service Permissions\n"<<endl;
        getServicePermissions(presentity,convert_list_to_string(ruleid_list));
	cout<<"Going to get person Permissions\n"<<endl;
        getPersonPermissions(presentity,convert_list_to_string(ruleid_list));
	cout<<"Completed Rule matching process\n"<<endl;
	
    }
    
    return filter;
}/* PrivacyRuleMatch :: getFilter(char *presentity, char *watcher, char *domain, char *scheme ,char *sphere) */


/* PrivacyRuleMatch :: getRuleidList(char *presentity,char *identity,char *domain, char *scheme,char *sphere) 
 * Finds the list of rules that match the watcher based on sphere,identity and validity and creates a list if ruleids.
 * Returns the list of matched ruleids 
 */
 
list<const char *> PrivacyRuleMatch :: getRuleidList(char *presentity,char *identity,char *domain, char *scheme,char *sphere) {
    
    cout<<"Getting matching ruleid list"<<endl;
    MYSQL_RES *result_identity;
    MYSQL_RES *result_ruleset;
    MYSQL_ROW identity_row;
    MYSQL_ROW ruleset_row;

    list<const char *> ruleid_list;
   
    result_identity = query->select_ruleidentity(presentity,identity,domain,scheme);
    while((identity_row = mysql_fetch_row(result_identity))!= NULL) {
        
        cout<<"\nSubquerying Ruleid : "<<identity_row[0]<<endl;
        result_ruleset = query->select_rulematch(presentity,identity_row[0],sphere);
        while((ruleset_row = mysql_fetch_row(result_ruleset)) != NULL) {
            ruleid_list.push_back(ruleset_row[0]);
        }
        mysql_free_result(result_ruleset);      
            
    }
    
    
   // print list contents
   cout<<"\nRuleid List Contents:\n"; 
   if(ruleid_list.size() == 0) {
       cout<<"No matching rules found !!"<<endl;
        	
   }
   else {
       copy(ruleid_list.begin(),ruleid_list.end(),ostream_iterator<const char *>(cout," ")); 
   }
   cout<<endl;
   mysql_free_result(result_identity);
   return ruleid_list;
                     
}/*PrivacyRuleMatch :: getRuleidList(char *presentity,char *domain,char *scheme,char *sphere) */ 

 
/* PrivacyRuleMatch :: getServicePermissions(char *presentity,string ruleid_set) 
 * Gets the service permissions based on the ruleids in the ruleid_set and assigns them in the filter object
 */
void PrivacyRuleMatch :: getServicePermissions(char *presentity,string ruleid_set) {

    MYSQL_RES *result_permission;
    MYSQL_ROW permission_row;
   
    
    result_permission = query->select_servicepermission(presentity,ruleid_set);
       
    while((permission_row = mysql_fetch_row(result_permission)) != NULL) {
         filter->service_permissions.insert(pair<const char *,const char *> (permission_row[0],permission_row[1])); 
    }
           
    mysql_free_result(result_permission); 
    cout<<endl; 
       
}/* PrivacyRuleMatch :: getServicePermissions(char *presentity,string ruleid_set) */
 

/* PrivacyRuleMatch :: getDevicePermissions(char *presentity,string ruleid_set) 
 * Gets the device permissions based on the ruleids in the ruleid_set and assigns them in the filter object
 */
void PrivacyRuleMatch :: getDevicePermissions(char *presentity,string ruleid_set) {
	MYSQL_RES *result_permission;       
        MYSQL_ROW permission_row;   
      
       result_permission = query->select_devicepermission(presentity,ruleid_set);
           
       while((permission_row = mysql_fetch_row(result_permission)) != NULL) {
	    filter->device_permissions.insert(pair<const char *,const char *> (permission_row[0],permission_row[1])); 
       }
       
       mysql_free_result(result_permission); 
          
       
}/* PrivacyRuleMatch :: getDevicePermissions(char *presentity,string ruleid_set) */


/* PrivacyRuleMatch :: getPersonPermissions(char *presentity,string ruleid_set) 
 * Gets the person permissions based on the ruleids in the ruleid_set and assigns them in the filter object
 */
void PrivacyRuleMatch :: getPersonPermissions(char *presentity,string ruleid_set) {

       MYSQL_RES *result_permission;       
       MYSQL_ROW permission_row;   
            
       result_permission = query->select_personpermission(presentity,ruleid_set);
       while((permission_row = mysql_fetch_row(result_permission)) != NULL) {
           filter->person_permissions.insert(pair<const char *,const char *> (permission_row[0],permission_row[1])); 
       }
       
       mysql_free_result(result_permission); 
       
}/* PrivacyRuleMatch :: getPersonPermissions(char *presentity,string ruleid_set) */
   
  

    	 
     




