/**
 * PrivacyRuleMatch.h: Fetches all matching rules for a watcher and combines them to a single rule
 * Copyright 2005 by Columbia University; all rights reserved
 */

#ifndef PRIVACY_RULEMATCH_H
#define PRIVACY_RULEMATCH_H

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <mysql.h>
#include <map>
#include <list>
#include <iterator>
#include <string>
#include <iostream>
#include "PolicyQuery.h"

using namespace std;

typedef struct match_result {
    char *identity;
    char *sphere;
    int  subscription_handling;
    bool provide_activities;
    bool provide_class;
    bool provide_device_id;
    bool provide_mood;
    bool provide_place_type;
    bool provide_place_is;
    bool provide_privacy;
    bool provide_relationship;
    bool provide_status_icon;
    bool provide_sphere;
    bool provide_time_offset;
    bool provide_all_services;
    bool provide_all_devices;
    bool provide_all_persons;
    int  provide_user_input;
    bool provide_note;
    bool provide_unknown_attribute;
    char *unknown_attribute_name;
    multimap<const char *,const char *> service_permissions;
    multimap<const char *,const char *> device_permissions;
    multimap<const char *,const char *> person_permissions;
}final_rule;

class PrivacyRuleMatch {

    public : 
        
        PrivacyRuleMatch();
         ~PrivacyRuleMatch();
        final_rule *getFilter(char *presentity,char *watcher,char *domain, char *scheme,char *sphere);
    
    protected:
    
        bool convert_to_boolean(char *str);
        list<const char *> getRuleidList(char *presentity,char *identity, char *domain,char *scheme,char *sphere);
        void getServicePermissions(char *presentity,string ruleid_set);
	void getDevicePermissions(char *presentity,string ruleid_set);
	void getPersonPermissions(char *presentity,string ruleid_set);
        void rule_combine(string ruleid_set);
        string convert_list_to_string(list<const char *> ruleid_list);
       
    
   private :    
        PolicyQuery *query;
        final_rule *filter;
               
    
};

#endif
